
# "knownPlain" constant derived from FastRandom(1990)
# https://github.com/ppy/osu-stream/blob/master/osu!stream/Helpers/osu!common/MapPackage.cs#L64
KNOWN_PLAIN = bytearray([
   0x55, 0xAA, 0x74, 0x10, 0x2B, 0x56, 0xB3, 0x9E,
   0x25, 0x9E, 0xFE, 0xB7, 0xBE, 0x06, 0xFC, 0xF2,
   0xB6, 0x3C, 0x6F, 0x47, 0x7E, 0x38, 0x69, 0x43,
   0x80, 0x89, 0x25, 0x00, 0xCC, 0xB6, 0xFE, 0x12,
   0xA9, 0xB2, 0x4A, 0x2C, 0x96, 0xD5, 0xEA, 0x26,
   0x42, 0x31, 0xAF, 0x0A, 0x0D, 0xAE, 0x00, 0xED,
   0xFE, 0x96, 0xA6, 0x94, 0x99, 0xA7, 0x90, 0xE4,
   0x68, 0xBF, 0xC6, 0x97, 0x5B, 0x1B, 0x5E, 0x7F
])

# A list of all allowed file extensions in an .osz package
# Did not cause any issues when testing on titanic
ALLOWED_FILE_EXTENSIONS = (
    "osu", "osz", "osb", "osk", "png", "mp3",
    "wav", "ogg", "jpg", "wmv", "flv", "flac",
    "avi", "ini",  "m4v", "mpg", "mov", "webm",
    "ogv", "mpeg", "3gp", "mkv", "mp4", "jpeg",
)
